#ifndef ___EVAL_PARAM_OF_RASER
#define ___EVAL_PARAM_OF_RASER

#include "definitions.h"
#include "raserOptions.h"
#include "tree.h"
#include "likelihoodComputation.h"
#include "likelihoodComputationSPvec.h"
#include "sequenceContainer.h"
#include "stochasticProcess.h"
#include "gammaDistribution.h"
#include "gammaDistributionFixedCategories.h"
#include "computeUpAlg.h"
#include "numRec.h"
#include "distributionPlusInvariant.h"
#include "distributionPlusCategory.h"


//The functor to evaluate parameters of raser for optimization
class evalParamOfRaser{
public:
	enum paramName {alphaRateDistr, alphaNuDistr, betaNuDistr, probNu0,probInfinity};
public:
	explicit evalParamOfRaser(const tree& et,
				const sequenceContainer& sc,
				const vector<stochasticProcess*> &spVec,
				paramName myparamName,
				const distributionPlusInvariant * nuDistr,
				const raserOptions::distrType nuDistrType,
				bool isFixedNuCategories,
				bool isEquallyProbableRateCategories,
				suffStatGlobalGam *ssc = NULL);
	MDOUBLE operator()(MDOUBLE param);
    virtual ~evalParamOfRaser();
	evalParamOfRaser(const evalParamOfRaser &other);
	void updateRateDistrAlpha(MDOUBLE param);
	void updateNuDistrAlpha(MDOUBLE param);
	void updateNuDistrBeta(MDOUBLE param);
	void updateProbNu0(MDOUBLE param);
	void updateProbInfinity(MDOUBLE param);
	void updateNullModelprobNu0(MDOUBLE param);
	void updateNullModelProbInfinity(MDOUBLE param);

private:
	paramName _paramName;
	const tree& _et;
	const sequenceContainer& _sc;
	vector<stochasticProcess*> _spVec;
	distributionPlusInvariant *_nuDistr;
	raserOptions::distrType _nuDistrType;
	bool _isFixedNuCategories;
	bool _isEquallyProbableRateCategories;
	suffStatGlobalGam *_ssc; // NULL indicates no pre-calculated up values.
};

#endif // ___EVAL_PARAM_OF_RASER
