// $Id: likeDistSPvec.h 953 2006-10-23 13:13:26Z eyalprivman $

#ifndef ___LIKE_DIST_SPVEC
#define ___LIKE_DIST_SPVEC

#include "definitions.h"
#include "countTableComponent.h"
#include "distanceMethod.h"
#include "stochasticProcess.h"
#include "distribution.h"
#include "logFile.h"
#include <cmath>
using namespace std;

class likeDistSPvec : public distanceMethod {
public:
	explicit likeDistSPvec(const vector<stochasticProcess*>& spVec, const distribution *distr,
					  const MDOUBLE toll =0.0001,
					  const MDOUBLE maxPairwiseDistance = 2.0) ;

	likeDistSPvec (const likeDistSPvec& other);
	virtual likeDistSPvec* clone() const {return new likeDistSPvec(*this);}
	virtual ~likeDistSPvec();


	// THIS FUNCTION DOES NOT RETURN THE LOG LIKELIHOOD IN RESQ, BUT RATHER "Q", THE CONTRIBUTION of this edge
    // TO THE EXPECTED LOG-LIKELIHOOD (SEE SEMPHY PAPER).
	// NEVERTHELESS, THE t that optimizes Q is the same t that optimizes log-likelihood.
	const MDOUBLE giveDistance(	const countTableComponentGam& ctc,
								MDOUBLE& resQ,
								const MDOUBLE initialGuess= 0.03) const; // initial guess
	
	
	// returns the estimated ML distance between the 2 sequences.
	// if score is given, it will be the log-likelihood.
	const MDOUBLE giveDistance(const sequence& s1,
						const sequence& s2,
						const vector<MDOUBLE>  * weights,
						MDOUBLE* score=NULL) const { return 1;}

	const MDOUBLE giveDistanceBrent(const countTableComponentGam& ctc, MDOUBLE& resL,
										   const MDOUBLE initialGuess) const;
	
private:
	vector<stochasticProcess*> _spVec;
	distribution *_distr;
	const MDOUBLE _toll;
	const MDOUBLE _maxPairwiseDistance;

};


class C_evalLikeDistSPvec{
private:
	const countTableComponentGam& _ctc;
	vector<stochasticProcess*> _spVec;
public:
	C_evalLikeDistSPvec(const countTableComponentGam& ctc,
					const vector<stochasticProcess*>& spVec);
	virtual ~C_evalLikeDistSPvec();

	MDOUBLE operator() (MDOUBLE dist) {
		const MDOUBLE epsilonPIJ = 1e-10;
		MDOUBLE sumL=0.0;
		for (int alph1=0; alph1 < _ctc.alphabetSize(); ++alph1){
			for (int alph2=0; alph2 <  _ctc.alphabetSize(); ++alph2){
				for (int categor = 0; categor<_spVec.size(); ++categor) {				
					MDOUBLE pij= _spVec[categor]->Pij_t(alph1,alph2,dist);
					if (pij<epsilonPIJ) pij = epsilonPIJ;//SEE REMARK (1) FOR EXPLANATION
					sumL += _ctc.getCounts(alph1,alph2,categor)*(log(pij)-log(_spVec[categor]->freq(alph2)));//*_sp.ratesProb(rateCategor);// removed.
				}
			}
		}
		return -sumL;
	};
};

// REMARK 1: THE LINE if if (pij<epsilonPIJ) pij = epsilonPIJ
// There are cases when i != j, and t!=0, and yet pij =0, because of numerical problems
// For these cases, it is easier to assume pij is very small, so that log-pij don't fly...

class C_evalLikeDist_d_SPvec { // derivative.
public:
  C_evalLikeDist_d_SPvec(const countTableComponentGam& ctc, const vector<stochasticProcess*>& spVec,
	  const distribution *distr);
	virtual ~C_evalLikeDist_d_SPvec();

private:
	const  countTableComponentGam& _ctc;
	vector<stochasticProcess*> _spVec;
	distribution *_distr;
public:
	MDOUBLE operator() (MDOUBLE dist);

};

#endif

