#include "posData.h"
#include <iostream>
#include "logFile.h"
#include <iomanip>

posData::posData(int pos_in_MSA, int pos_in_RefSeq,int refSeqContent) :
_pos_in_MSA(pos_in_MSA),
_pos_in_RefSeq(pos_in_RefSeq),
_refSeqContent(refSeqContent),
_postValRateShift(-1)
{}


void posData::addNodeData(int nodeId,bool isLeaf, MDOUBLE probAcceleration,MDOUBLE probDeceleration) {
	_nodesData.push_back(nodeData(nodeId,isLeaf,probAcceleration,probDeceleration));
}

void posData::clearNodesData() {
	_nodesData.clear();
}

void posData::addPosteriorProb(MDOUBLE posteriorProb) {
	_posteriorProbs.push_back(posteriorProb);
}
void posData::clearPosteriorProbs() {
	_posteriorProbs.clear();
}

void posData::printPostVal(ostream &out, const alphabet* alph,MDOUBLE cutoff){

	out<<_pos_in_RefSeq+1 <<"\t"<<alph->fromInt(_refSeqContent)<<"\t";
	LOG(5,<<_pos_in_RefSeq+1 <<"\t"<<alph->fromInt(_refSeqContent)<<" \t")
	for (int cat = 0; cat < _posteriorProbs.size(); ++cat){
		LOG(5,<<_posteriorProbs[cat]<<"\t");
	}
	out<<_postValRateShift;
	if (_postValRateShift>=cutoff)
		out<<"\t*";
	out << endl;
	LOG(5,<<endl);
}

void posData::sortNodesData() {
	if (_nodesData.size() < 1) return;
	sort(_nodesData.begin(),_nodesData.end());
	reverse(_nodesData.begin(),_nodesData.end());
}

// assumes that _posteriorProbs is filled
void posData::calcPostValRateShift(int numRateShiftCategories) {
	_postValRateShift=0.0;
	for (int i=0; i<numRateShiftCategories; ++i)
		_postValRateShift+=_posteriorProbs[i];
}

//	out << "pos" << '\t' << "node" << '\t' << "probAcc" << '\t' << "probDec" << '\t' << "probShift" << endl;
void posData::printNodesData(ostream &out, int numNodes4pos,MDOUBLE nodes_cutoff) const 
{
	if (_nodesData.size() < 1) return;

	int counterPrinted = 0; // how many nodes printed till now
	int counter = 0; // nodes
	while (counterPrinted < numNodes4pos){
	    if (!(_nodesData[counter].isLeaf())){
		out << left << setw(5) << _pos_in_RefSeq+1;
		out << _nodesData[counter] ;
		if ((_nodesData[counter].getProbAcceleration()>nodes_cutoff) || (_nodesData[counter].getProbDeceleration()>nodes_cutoff) )
		{
		    
		    out<<"\t*";
		}
		out<<endl;
		counterPrinted++;
	    }
	    counter++;
	}
}
