/********************************************************************************************
raserOptions - a class that contains all the parameters.
	use the 'Parameters' class to read info from txt file.
	initDefault. (+Parameters::addParameter)
	getParamsFromFile. ->with alterations of defults for consistancy
	verifyConsistParams.
*********************************************************************************************/
#include "raserOptions.h"
#include "errorMsg.h"
#include "someUtil.h"
#include "Parameters.h"
#include <iostream>
#include <cmath>

using namespace std;

raserOptions::~raserOptions(){}
/********************************************************************************************
*********************************************************************************************/
void raserOptions::initOptions(const string& paramFileName)
{	
	initDefault();
	getParamsFromFile(paramFileName);
	verifyConsistParams();
}

/********************************************************************************************
initDefault
*********************************************************************************************/
void raserOptions::initDefault()
{
// all the default values are stored in the raserOptions:: static members
//################### Basic parameters:
// input (general)
    _inSeqFile = "";
    _inTreeFile = "";
	_inQuerySeq = "";
    _numBranchesToPrint = 3;
	_specificNodeForStochasticMapping = -1; 

// output
	_logFile = "raser.log";   
	 _verboseLevel = 5;
    _outTreeFile = "raser.tree";
	_outTreeFileWithBranchesNames = "raser.tree.namesBS.ph";
    _outResFile = "raser.res";
	_outNodesResFile = "raser.res.nodesPosteriorProbs";

//################################################## Model params    
    _modelName = raserOptions::jtt;
    _nuDistrType = generalGammaInfDist;
    _numOfCategoriesForRateDistr = 4;
    _numOfCategoriesForNuDistr = 5;
	_onlyPrintNodeIDs = false;
	_fixedProbNu0 = false;
    _initProbNu0 = 0.5;
    _fixedAlphaRateDistr = false;
    _initAlphaRateDistr = 1.0;
    _fixedAlphaNuDistr = false;
    _initAlphaNuDistr  = 1.0;
    _fixedBetaNuDistr = false;
    _initBetaNuDistr = 1.0;
    _fixedInfinityProb = false;
    _initInfinityProb = 0.25; 
    _posteriorCutoffSites = 0.95;
    _posteriorCutoffNodes = 0.5;
    _useFixedCategoriesForNuDistribution = true;
    _useFixedCategoriesForRateDistribution = true;
    _useLaguerreRateDistribution = false;
    _useNullModel = false;
//    _upperBoundNuForFixedcategories = 0.0; // signifies that the upper-bound was not set
//    _upperBoundRateForFixedcategories = 0.0; // signifies that the upper-bound was not set
    _infinityNu = 20;
    _alphabetSize = 20; //default is amino-acid
    _bblOpt = true; // by default perform BBL optimization
    _manyStartPointsOpt = true; // by default perform many starting points for the optimization
    _epsilonLikelihood = 0.1;
    _numBBLiterations = 5; //bbl iterations

// all the parameters are added to the static: ParamList paramList (vector<Parameter>);	
	Parameters::addParameter("_inSeqFile", _inSeqFile);
	Parameters::addParameter("_inTreeFile", _inTreeFile);
	Parameters::addParameter("_inQuerySeq", _inQuerySeq);
	Parameters::addParameter("_numBranchesToPrint", _numBranchesToPrint);
	Parameters::addParameter("_specificNodeForStochasticMapping", _specificNodeForStochasticMapping);
	Parameters::addParameter("_logFile", _logFile);
	Parameters::addParameter("_verboseLevel", _verboseLevel);
	Parameters::addParameter("_outTreeFile", _outTreeFile);
	Parameters::addParameter("_outTreeFileWithBranchesNames", _outTreeFileWithBranchesNames);
	Parameters::addParameter("_outResFile", _outResFile);
	Parameters::addParameter("_outNodesResFile", _outNodesResFile);
	Parameters::addParameter("_modelName", getModelNameType(_modelName));
	Parameters::addParameter("_nuDistrType", getDistributionType(_nuDistrType));
	Parameters::addParameter("_numOfCategoriesForRateDistr", _numOfCategoriesForRateDistr);
	Parameters::addParameter("_numOfCategoriesForNuDistr", _numOfCategoriesForNuDistr);
	Parameters::addParameter("_fixedProbNu0", _fixedProbNu0);
	Parameters::addParameter("_onlyPrintNodeIDs", _onlyPrintNodeIDs);
	Parameters::addParameter("_initProbNu0", _initProbNu0);
	Parameters::addParameter("_fixedAlphaRateDistr", _fixedAlphaRateDistr);
	Parameters::addParameter("_initAlphaRateDistr", _initAlphaRateDistr);
	Parameters::addParameter("_fixedAlphaNuDistr", _fixedAlphaNuDistr);
	Parameters::addParameter("_initAlphaNuDistr", _initAlphaNuDistr);
	Parameters::addParameter("_fixedBetaNuDistr", _fixedBetaNuDistr);
	Parameters::addParameter("_initBetaNuDistr", _initBetaNuDistr);
	Parameters::addParameter("_fixedInfinityProb", _fixedInfinityProb);
	Parameters::addParameter("_initInfinityProb", _initInfinityProb);
	Parameters::addParameter("_posteriorCutoffSites", _posteriorCutoffSites);
	Parameters::addParameter("_posteriorCutoffNodes", _posteriorCutoffNodes);
	Parameters::addParameter("_useFixedCategoriesForNuDistribution", _useFixedCategoriesForNuDistribution);
	Parameters::addParameter("_useFixedCategoriesForRateDistribution", _useFixedCategoriesForRateDistribution);
	Parameters::addParameter("_useLaguerreRateDistribution", _useLaguerreRateDistribution);
	Parameters::addParameter("_useNullModel", _useNullModel);
//	Parameters::addParameter("_upperBoundNuForFixedcategories", _upperBoundNuForFixedcategories);
//	Parameters::addParameter("_upperBoundRateForFixedcategories", _upperBoundRateForFixedcategories);
	Parameters::addParameter("_infinityNu", _infinityNu);
	Parameters::addParameter("_alphabetSize", _alphabetSize);
	Parameters::addParameter("_bblOpt", _bblOpt);
	Parameters::addParameter("_manyStartPointsOpt", _manyStartPointsOpt);
	Parameters::addParameter("_epsilonLikelihood", _epsilonLikelihood);
	Parameters::addParameter("_numBBLiterations", _numBBLiterations);

}
/********************************************************************************************
readParameters
*********************************************************************************************/
void raserOptions::readParameters(const string& paramFileName)
{
	ifstream params(paramFileName.c_str());
	if(params.good())
		Parameters::readParameters(params);		// only place where params are read, updateParameter(paramName, param.c_str()) used
	params.close();
}

/********************************************************************************************
getParamsFromFile
*********************************************************************************************/
void raserOptions::getParamsFromFile(const string& paramFileName)
{	
	readParameters(paramFileName);
	_inSeqFile = Parameters::getString("_inSeqFile");
	_inTreeFile = Parameters::getString("_inTreeFile");
	_inQuerySeq = Parameters::getString("_inQuerySeq");
	_numBranchesToPrint = Parameters::getInt("_numBranchesToPrint");
	_specificNodeForStochasticMapping = Parameters::getInt("_specificNodeForStochasticMapping");
	
	_logFile = Parameters::getString("_logFile");
	_verboseLevel = Parameters::getInt("_verboseLevel");
	_outTreeFile = Parameters::getString("_outTreeFile");
	_outTreeFileWithBranchesNames = Parameters::getString("_outTreeFileWithBranchesNames");
	_outResFile = Parameters::getString("_outResFile");
	_outNodesResFile = Parameters::getString("_outNodesResFile");

	_modelName = getModelNameType(Parameters::getString("_modelName"));

	// Currently we don't want the users to change the nu ditribution type,
	// so we put the next line in a remark.
	//_nuDistrType = getDistributionType(Parameters::getString("_nuDistrType"));

	_numOfCategoriesForRateDistr = Parameters::getInt("_numOfCategoriesForRateDistr");
	_numOfCategoriesForNuDistr = Parameters::getInt("_numOfCategoriesForNuDistr");

	_fixedProbNu0 = (Parameters::getInt("_fixedProbNu0") == 1) ? true : false;
	_onlyPrintNodeIDs = (Parameters::getInt("_onlyPrintNodeIDs") == 1) ? true : false;
	_initProbNu0 = Parameters::getFloat("_initProbNu0");
	_fixedAlphaRateDistr = (Parameters::getInt("_fixedAlphaRateDistr") == 1) ? true : false;
	_initAlphaRateDistr = Parameters::getFloat("_initAlphaRateDistr");
	_fixedAlphaNuDistr = (Parameters::getInt("_fixedAlphaNuDistr") == 1) ? true : false;
	_initAlphaNuDistr = Parameters::getFloat("_initAlphaNuDistr");
	_fixedBetaNuDistr = (Parameters::getInt("_fixedBetaNuDistr") == 1) ? true : false;
	_initBetaNuDistr = Parameters::getFloat("_initBetaNuDistr");
	_fixedInfinityProb = (Parameters::getInt("_fixedInfinityProb") == 1) ? true : false;
	_initInfinityProb = Parameters::getFloat("_initInfinityProb");
	_posteriorCutoffSites = Parameters::getFloat("_posteriorCutoffSites"); 
	_posteriorCutoffNodes = Parameters::getFloat("_posteriorCutoffNodes");

//	_useFixedCategoriesForNuDistribution = (Parameters::getInt("_useFixedCategoriesForNuDistribution") == 1) ? true : false;
//	_useFixedCategoriesForRateDistribution = (Parameters::getInt("_useFixedCategoriesForRateDistribution") == 1) ? true : false;
//	_useLaguerreRateDistribution = (Parameters::getInt("_useLaguerreRateDistribution") == 1) ? true : false;
	_useNullModel = (Parameters::getInt("_useNullModel") == 1) ? true : false;

//	_upperBoundNuForFixedcategories = Parameters::getFloat("_upperBoundNuForFixedcategories"); 
//	_upperBoundRateForFixedcategories = Parameters::getFloat("_upperBoundRateForFixedcategories");
//	_infinityNu = Parameters::getFloat("_infinityNu");

	_alphabetSize = Parameters::getInt("_alphabetSize");
	_bblOpt = (Parameters::getInt("_bblOpt") == 1) ? true : false;
	_manyStartPointsOpt = (Parameters::getInt("_manyStartPointsOpt") == 1) ? true : false;
	_epsilonLikelihood = Parameters::getFloat("_epsilonLikelihood");
	_numBBLiterations = Parameters::getInt("_numBBLiterations");
}

/********************************************************************************************
*********************************************************************************************/
string raserOptions::getDistributionType(distrType type) 
{
	string res = "";
	switch (type)
	{
	case gammaDist:
		res = "gammaDist";
		break;
	case generalGammaDist:
		res = "generalGammaDist";
		break;
	case betaDist:
		res = "betaDist";
		break;
	case betaInfDist:
		res = "betaInfDist";
		break;
	case gammaInfDist:
		res = "gammaInfDist";
		break;
	case generalGammaInfDist:
		res = "generalGammaInfDist";
		break;
	default:
		errorMsg::reportError("ERROR in raserOptions::getDistributionType : unknown distributionType");
	}
	return res;
}

raserOptions::distrType raserOptions::getDistributionType(const string& str) 
{
	if (str == "gammaDist")
		return gammaDist;
	if (str == "generalGammaDist")
		return generalGammaDist;
	if (str == "betaDist")
		return betaDist;
	if (str == "betaInfDist")
		return betaInfDist;
	if (str == "gammaInfDist")
		return gammaInfDist;
	if (str == "generalGammaInfDist")
		return generalGammaInfDist;
	else
		errorMsg::reportError("ERROR in raserOptions::getDistributionType : unknown type");
	return generalGammaInfDist;
}
/********************************************************************************************
*********************************************************************************************/
string raserOptions::getModelNameType(modelNameType type) 
{
	string res = "";
	switch (type)
	{
	case rev:
		res = "rev";
		break;
	case jtt:
		res = "jtt";
		break;
	case day:
		res = "day";
		break;
	case aajc:
		res = "aajc";
		break;
	case nucjc:
		res = "nucjc";
		break;
	case wag:
		res = "wag";
		break;
	case cprev:
		res = "cprev";
		break;
	case HIVb:
		res = "HIVb";
		break;
	case HIVw:
		res = "HIVw";
		break;
	default:
		errorMsg::reportError("ERROR in raserOptions::getModelNameType : unknown distributionType");
	}
	return res;
}

raserOptions::modelNameType raserOptions::getModelNameType(const string& str) 
{
	if (str == "rev")
		return rev;
	if (str == "jtt")
		return jtt;
	if (str == "day")
		return day;
	if (str == "aajc")
		return aajc;
	if (str == "nucjc")
		return nucjc;
	if (str == "wag")
		return wag;
	if (str == "cprev")
		return cprev;
	if (str == "HIVb")
		return HIVb;
	if (str == "HIVw")
		return HIVw;
	else
		errorMsg::reportError("ERROR in raserOptions::getModelNameType : unknown type");
	return jtt;
}

/********************************************************************************************
*********************************************************************************************/
void raserOptions::verifyConsistParams()
{
	if (_inSeqFile.size() == 0)	{
		errorMsg::reportError("The input sequence file is missing");
	}
	if (_fixedInfinityProb) {
		if (!_fixedProbNu0)
			errorMsg::reportError("The program does not support fixing only prob(nu=inf) without prob(nu=0)");
	}
	if (_fixedProbNu0) {
		if (!_fixedInfinityProb)
			errorMsg::reportError("The program does not support fixing only prob(nu=0) without prob(nu=inf)");
	}
	
	if (getModelNameType(_modelName)=="nucjc") {
		_alphabetSize=4;
		Parameters::updateParameter("_alphabetSize","4");
	}	

	if (_useNullModel) {
		_numOfCategoriesForNuDistr=1;
		Parameters::updateParameter("_numOfCategoriesForNuDistr","1");
		_manyStartPointsOpt=false; // no need for multiple starting points
		Parameters::updateParameter("_manyStartPointsOpt","0");
		LOGnOUT(5,<<"Null model used, using only one starting point (_manyStartPointsOpt=false)"<<endl);
	}
	// if all parameters are fixed - no use for multiple starting points
	if (_fixedProbNu0) {
		if (_fixedInfinityProb){
			if (_fixedAlphaRateDistr) {
				if (_fixedAlphaNuDistr) {
					if (_fixedBetaNuDistr) {
						if (_manyStartPointsOpt) {
							_manyStartPointsOpt=false;
							Parameters::updateParameter("_manyStartPointsOpt","0");
							LOGnOUT(5,<<"All parameters are fixed, using only one starting point (_manyStartPointsOpt=false)"<<endl);
						}
					}
				}
			}
		}
	}
}

/********************************************************************************************
*********************************************************************************************/

void raserOptions::printOptionParameters() {
	LOGnOUT(5,<<"\n ---------------------- THE PARAMETERS FOR THE RUN------------------------"<<endl);
	LOGnOUT(5,<<"input alignment file is:    "<<_inSeqFile <<endl);
	if (!_onlyPrintNodeIDs) {
		LOGnOUT(5,<<"Model is "<<getModelNameType(_modelName)<<endl);
		LOGnOUT(5,<<"Epsilon for likelihood (precision level):          "<<_epsilonLikelihood <<endl);
		LOGnOUT(5,<<"reference sequence is:          "<<(_inQuerySeq==""?"not given":_inQuerySeq) <<endl);
	}
	LOGnOUT(5,<<"input tree file is:             "<<_inTreeFile <<endl);
	if (!_onlyPrintNodeIDs) {
		LOGnOUT(5,<<"branch length optimization:     "<<(_bblOpt?"do":"don't do")<<endl);
		LOGnOUT(5,<<"use NULL model:          "<<(_useNullModel?"yes":"no")<<endl);
		LOGnOUT(5,<<"use fixed categories for nu distribution: "<<(_useFixedCategoriesForNuDistribution?"yes":"no")<<endl);
		LOGnOUT(5,<<"use fixed categories for rate distribution: "<<(_useFixedCategoriesForRateDistribution?"yes":"no")<<endl);
//		LOGnOUT(5,<<"use Guass-Laguerre quadrature method for rate distribution: "<<(_useLaguerreRateDistribution?"yes":"no")<<endl);
//		if (_upperBoundNuForFixedcategories>0) {
//			LOG(5,<<"upper bound on nu value:"<<_upperBoundNuForFixedcategories);
//		}
//		if (_upperBoundRateForFixedcategories>0) {
//			LOG(5,<<"upper bound on rate value:"<<_upperBoundRateForFixedcategories);
//		}
		LOGnOUT(5,<<"initial alpha for nu distr:     "<<_initAlphaNuDistr<<endl);
		LOGnOUT(5,<<"alpha nu distr optimization:     "<<(_fixedAlphaNuDistr?"don't do":"do")<<endl);
		LOGnOUT(5,<<"initial alpha for rate distr:     "<<_initAlphaRateDistr<<endl);
		LOGnOUT(5,<<"alpha rate distr optimization:     "<<(_fixedAlphaRateDistr?"don't do":"do")<<endl);
		//if ((_nuDistrType != gammaDist) && (_nuDistrType != gammaInfDist)){
		LOGnOUT(5,<<"initial beta for nu distr:     "<<_initBetaNuDistr<<endl);
		LOGnOUT(5,<<"beta nu distr optimization:     "<<(_fixedBetaNuDistr?"don't do":"do")<<endl);
		//}	
		LOGnOUT(5,<<"initial prob(nu=0):     "<<_initProbNu0<<endl);
		LOGnOUT(5,<<"prob(nu=0) optimization:     "<<(_fixedProbNu0?"don't do":"do")<<endl);
		LOGnOUT(5,<<"initial prob(nu=inf):     "<<_initInfinityProb<<endl);
		LOGnOUT(5,<<"prob(nu=inf) optimization:     "<<(_fixedInfinityProb?"don't do":"do")<<endl);
	}
	LOGnOUT(5,<<"output log file is:             "<< _logFile<<endl);
	if (!_onlyPrintNodeIDs) {
		LOGnOUT(5,<<"output tree file is:            "<<_outTreeFile <<endl);            
	}
	LOGnOUT(5,<<"output tree with branches names file is:            "<<_outTreeFileWithBranchesNames <<endl);            

	if (!_onlyPrintNodeIDs) {
		LOGnOUT(5,<<"output nu for site file is:  "<<_outResFile <<endl);
		LOGnOUT(5,<<"output nu for nodes file is:  "<<_outNodesResFile <<endl);
	} else {
		LOGnOUT(5,<<"Only print Node IDs without running RASER"<<endl);
	}
}
