#include "shiftNodes4site.h"
#include "simulateRateShiftJumps.h"

shiftNodes4site::shiftNodes4site(const tree& et,
					  const sequenceContainer& scSSRV,const sequenceContainer& baseSc,
					  const vector <stochasticProcess*>& spVec, const distribution* nuDistr,
					  const sequence &refseq):
_et(et),
_scSSRV(scSSRV),
_baseSc(baseSc),
_spVec(spVec),
_nuDistr(nuDistr),
_refSeq(refseq)
{
	init();
}

void shiftNodes4site::init()
{
	_cup.allocatePlace(_nuDistr->categories(), _et.getNodesNum(), _scSSRV.alphabetSize());
	_cdown.allocatePlace(_nuDistr->categories(), _et.getNodesNum(), _scSSRV.alphabetSize());
}


void shiftNodes4site::computeSignificantBranches(vector<posData>& positionsData,MDOUBLE cutoff,
												 computePijGam& pij, tree::nodeP myNode /*= NULL*/)
{
	LOG(8,<< "shiftNodes4site::computeSignificantBranches" << endl);

	if (myNode) {
		vector <tree::nodeP> nodes2calc;
		nodes2calc.push_back(myNode); //debug
		fillBranchesData(positionsData,cutoff,pij,nodes2calc);
	}
	else {
		fillBranchesData(positionsData,cutoff,pij);
	}
	
	// sort branches data
	vector<posData>::iterator posDataItr;
	for (posDataItr=positionsData.begin(); posDataItr != positionsData.end(); ++posDataItr) {
		posDataItr->sortNodesData();
	}
}

void shiftNodes4site::fillBranchesData(vector<posData>& positionsData,MDOUBLE cutoff,computePijGam& pij, vector <tree::nodeP> nodesToCalculate){
	// call simulations
	vector <simulateRateShiftJumps*> v_simJumps;
	for (int i = 0; i<_nuDistr->categories()-2; ++i) {
		
		LOG(5,<<"running simulations for nu category ="<<_nuDistr->rates(i)<<endl);
		cout<<"running simulations for nu category "<<_nuDistr->rates(i)<<endl;
		simulateRateShiftJumps *sims = new simulateRateShiftJumps(_et,*(_spVec[i]),_spVec[i]->alphabetSize());
		sims->runSimulation(10000,nodesToCalculate);
		v_simJumps.push_back(sims);
	}

	treeIterDownTopConst tIt(_et);
	int letter,fatherLetter;
	const mulAlphabet* alphabetSSRV = static_cast<const mulAlphabet*>(_scSSRV.getAlphabet());
	vector<posData>::iterator positionsDataItr;
	for (positionsDataItr=positionsData.begin(); positionsDataItr != positionsData.end(); ++positionsDataItr) { 
		if (positionsDataItr->getPostValRateShift() < cutoff)
			continue;
		if (positionsDataItr->getPosInRefSeq() < 0) // we seek for branches of the refSeq significant positions only
			continue;
		calcPosUpAndDown(*positionsDataItr,pij);
		vector<doubleRep> accProbs(_et.getNodesNum(),0.0); // for each node
		vector<doubleRep> decProbs(_et.getNodesNum(),0.0); // for each node
		vector<doubleRep> noShiftProbs(_et.getNodesNum(),0.0); // for each node
		doubleRep likelihoodPos(0.0),val(0.0);
// call getLofPos: _spVec[0] is given for use of freqs of letters which are the same across all sp's
		likelihoodPos = likelihoodComputationSPvec::getProbOfPosUpIsFilledSelectionGam(positionsDataItr->getPosInMSA(),_et,_scSSRV,*(_spVec[0]),_cup,_nuDistr);
		for (int nuCategor=0; nuCategor < _nuDistr->categories()-2; ++nuCategor) {
			for(letter=0; letter<alphabetSSRV->size();++letter) {
				for (int i = 0; i< nodesToCalculate.size(); ++i) {
					int nodeId = nodesToCalculate[i]->id();
					//tree::nodeP mynode = _et.findNodeById(nodeId);
					string nodeName = nodesToCalculate[i]->name();
					for (fatherLetter=0; fatherLetter<alphabetSSRV->size();++fatherLetter) {
 						val=calcVal(nodeId,nuCategor,pij[nuCategor],letter,fatherLetter);
						accProbs[nodeId]+= val * v_simJumps[nuCategor]->getProb(nodeName,fatherLetter,letter,mulAlphabet::acceleration);
						decProbs[nodeId]+= val * v_simJumps[nuCategor]->getProb(nodeName,fatherLetter,letter,mulAlphabet::deceleration);
						//cout<<decProbs[nodeId]<<endl;//debug
					} 
				}
			}
		}
		
		for (int i = 0; i< nodesToCalculate.size(); ++i) {
			
			doubleRep probAcceleration = accProbs[nodesToCalculate[i]->id()] / likelihoodPos;
			doubleRep probDeceleration = decProbs[nodesToCalculate[i]->id()] / likelihoodPos;
			// WARNING: assuming that the nodes in nodesToCalculatewith are not a leaf!!!
			positionsDataItr->addNodeData(nodesToCalculate[i]->id(),false,convert(probAcceleration),convert(probDeceleration));		
			
			//LOG(5,<<"probability of dec for position "<<positionsDataItr->getPosInRefSeq()<<" = "<<probDeceleration<<endl);
		}
	}	

	//delete
	for (int i = 0; i<_nuDistr->categories()-2; ++i) {
		delete v_simJumps[i];
	}
}


void shiftNodes4site::fillBranchesData(vector<posData>& positionsData,MDOUBLE cutoff,computePijGam& pij)
{
	treeIterDownTopConst tIt(_et);
	int letter,fatherLetter;
	const mulAlphabet* alphabetSSRV = static_cast<const mulAlphabet*>(_scSSRV.getAlphabet());
	vector<posData>::iterator positionsDataItr;
	for (positionsDataItr=positionsData.begin(); positionsDataItr != positionsData.end(); ++positionsDataItr) { 
		if (positionsDataItr->getPostValRateShift() < cutoff)
			continue;
		if (positionsDataItr->getPosInRefSeq() < 0) // we seek for branches of the refSeq significant positions only
			continue;
		calcPosUpAndDown(*positionsDataItr,pij);
		vector<doubleRep> accProbs(_et.getNodesNum(),0.0); // for each node
		vector<doubleRep> decProbs(_et.getNodesNum(),0.0); // for each node
		vector<doubleRep> noShiftProbs(_et.getNodesNum(),0.0); // for each node
		doubleRep likelihoodPos(0.0),val(0.0);
		for (int nuCategor=0; nuCategor < _spVec.size(); ++nuCategor) {
			for(letter=0; letter<alphabetSSRV->size();++letter) {
				for (tree::nodeP mynode = tIt.first(); mynode != tIt.end(); mynode = tIt.next()) {
					if (mynode->isRoot() == false)	{
						for (fatherLetter=0; fatherLetter<alphabetSSRV->size();++fatherLetter) {
							val=calcVal(mynode->id(),nuCategor,pij[nuCategor],letter,fatherLetter);
							int categoryCompare = alphabetSSRV->compareCategories(fatherLetter,letter);
							switch (categoryCompare) {
								case -1: decProbs[mynode->id()] += val ; break;
								case 1: accProbs[mynode->id()] += val ; break;
								case 0: noShiftProbs[mynode->id()] += val ; break;
							}
						} 
					}
					else {
						likelihoodPos += _cup.get(nuCategor,mynode->id(),letter) * 
							_spVec[nuCategor]->freq(letter)* _nuDistr->ratesProb(nuCategor);
					}
				}
			}
		}
		
		for (tree::nodeP mynode = tIt.first(); mynode != tIt.end(); mynode = tIt.next()) {
			if (mynode->isRoot() == false)	{
				doubleRep probAcceleration = accProbs[mynode->id()] / likelihoodPos;
				doubleRep probDeceleration = decProbs[mynode->id()] / likelihoodPos;
				doubleRep probNoRateShift = noShiftProbs[mynode->id()] / likelihoodPos;
				positionsDataItr->addNodeData(mynode->id(),mynode->isLeaf(),convert(probAcceleration),convert(probDeceleration));		
				// check1
				MDOUBLE sumProbPos= convert(probDeceleration)+convert(probAcceleration)+convert(probNoRateShift);
				MDOUBLE epsilon = 0.000001;
				if ((sumProbPos + epsilon < 1.0) || (sumProbPos > 1.0 + epsilon))
				{
					string err = "shiftNodes4site::fillBranchesData ERROR !!! sumProbPos = ";
					err+=double2string(sumProbPos);
					errorMsg::reportError(err); 
				}

			}
		}
	}	
}


// #### In practice, we don't need the up and down of the the ASRV and homogeneous categories (nu=0, nu=inf)
void shiftNodes4site::calcPosUpAndDown(const posData& positionData,computePijGam& pij)
{
	computeUpAlg cupAlg;
	computeDownAlg cdownAlg;
	int pos_in_MSA = positionData.getPosInMSA();
	for (int categor = 0; categor < _spVec.size(); ++categor) {
		cupAlg.fillComputeUp(_et,_scSSRV,pos_in_MSA,pij[categor],_cup[categor]);
		cdownAlg.fillComputeDown(_et,_scSSRV,pos_in_MSA,pij[categor],_cdown[categor],_cup[categor]);
	}

}


doubleRep shiftNodes4site::calcVal(int id, int nuCategor, computePijHom& nuCategorPij,int letter, int fatherLetter) const
{
	doubleRep downVal(0.0),upVal(0.0),val(0.0);
	MDOUBLE pijVal(0.0),freqVal(0.0);

	freqVal = _spVec[nuCategor]->freq(fatherLetter);
	MDOUBLE categorVal = _nuDistr->ratesProb(nuCategor);
	pijVal = nuCategorPij.getPij(id,fatherLetter,letter);
	upVal = _cup[nuCategor].get(id,letter);
	// note: this is the downAlg result for the father of mynode
	downVal = _cdown[nuCategor].get(id,fatherLetter); 
	val= freqVal*categorVal*upVal*downVal*pijVal;
	return val;
}
